#!/usr/bin/env python
# -*- coding: utf-8 -*-

#  SPDX-FileCopyrightText: 2019-2022 Alliander N.V.
#  SPDX-License-Identifier: MPL-2.0

from fastapi import APIRouter

router = APIRouter()


# TODO: implement API status retrieval
def get_api_status():
    pass
